@@ -1,14 +1,35 @@
 #ifndef __FatTree_h__
 #define __FatTree_h__
 
-#include "dextv.h"
+#include "dex2tv.h"
+#include "avtv.h"
 
-struct OSPV_fatindex : OSPV_Generic {
+struct OSPV_fattree_av : OSPV_Generic {
   static os_typespec *get_os_typespec();
-  dexXPVTV fi_tv;
+  XPVTV ary;
+  OSPV_fattree_av();
+  ~OSPV_fattree_av();
+  virtual char *os_class(STRLEN *);
+  virtual char *rep_class(STRLEN *);
+  virtual int get_perl_type();
+  virtual OSSV *avx(int xx);
+  virtual OSSV *FETCH(SV *xx);
+  virtual OSSV *STORE(SV *xx, SV *value);
+  virtual SV *Pop();
+  virtual void Push(SV *);
+  virtual void CLEAR();
+  virtual int _count();
+  virtual OSSV *traverse(char *keyish);
+  virtual void XSHARE(int on);
+};
+
+
+struct OSPV_fatindex2 : OSPV_Generic {
+  static os_typespec *get_os_typespec();
+  XPVTV tv;
   OSSVPV *conf_slot;
-  OSPV_fatindex();
-  virtual ~OSPV_fatindex();
+  OSPV_fatindex2();
+  virtual ~OSPV_fatindex2();
   virtual char *os_class(STRLEN *);
   virtual char *rep_class(STRLEN *len);
   virtual int get_perl_type();
@@ -21,11 +42,12 @@ struct OSPV_fatindex : OSPV_Generic {
   virtual OSSVPV *new_cursor(os_segment *seg);
 };
 
-struct OSPV_fatindex_cs : OSPV_Cursor2 {
+struct OSPV_fatindex2_cs : OSPV_Cursor2 {
   static os_typespec *get_os_typespec();
-  dexXPVTC fi_tc;
-  OSPV_fatindex_cs(OSPV_fatindex *_at);
-  virtual ~OSPV_fatindex_cs();
+  OSPV_fatindex2 *myfocus;
+  XPVTC tc;
+  OSPV_fatindex2_cs(OSPV_fatindex2 *_at);
+  virtual ~OSPV_fatindex2_cs();
   virtual OSSVPV *focus();
   virtual void moveto(I32);
   virtual void step(I32 delta);
