@@ -20,7 +20,7 @@ struct OSPV_hvarray : OSPV_Generic {
   virtual ~OSPV_hvarray();
   static void _boot(HV *);
   virtual OSSVPV *new_cursor(os_segment *seg);
-  virtual ossv_bridge *new_bridge();
+  virtual ospv_bridge *new_bridge();
   virtual char *os_class(STRLEN *);
   virtual int get_perl_type();
   int index_of(char *key);
@@ -31,8 +31,8 @@ struct OSPV_hvarray : OSPV_Generic {
   virtual void DELETE(char *key);
   virtual void CLEAR();
   virtual int EXISTS(char *key);
-  virtual SV *FIRST(ossv_bridge*);
-  virtual SV *NEXT(ossv_bridge*);
+  virtual SV *FIRST(ospv_bridge*);
+  virtual SV *NEXT(ospv_bridge*);
   virtual double _percent_filled();
   virtual int _count();
   virtual OSSV *traverse(char *keyish);
@@ -55,14 +55,14 @@ struct OSPV_setarray : OSPV_Generic {
   virtual ~OSPV_setarray();
   static void _boot(HV *);
   virtual OSSVPV *new_cursor(os_segment *seg);
-  virtual ossv_bridge *new_bridge();
+  virtual ospv_bridge *new_bridge();
   virtual char *os_class(STRLEN *);
   int first(int start);
   virtual void set_add(SV *);
   virtual int set_contains(SV *);
   virtual void set_rm(SV *);
-  virtual SV *FIRST(ossv_bridge*);
-  virtual SV *NEXT(ossv_bridge*);
+  virtual SV *FIRST(ospv_bridge*);
+  virtual SV *NEXT(ospv_bridge*);
   virtual void CLEAR();
   virtual double _percent_filled();
   virtual int _count();
@@ -88,14 +88,14 @@ struct OSPV_sethash : OSPV_Generic {
   OSPV_sethash(os_unsigned_int32 size);
   virtual ~OSPV_sethash();
   static void _boot(HV *);
-  virtual ossv_bridge *new_bridge();
+  virtual ospv_bridge *new_bridge();
   virtual OSSVPV *new_cursor(os_segment *seg);
   virtual char *os_class(STRLEN *);
   virtual void set_add(SV *);
   virtual int set_contains(SV *);
   virtual void set_rm(SV *);
-  virtual SV *FIRST(ossv_bridge*);
-  virtual SV *NEXT(ossv_bridge*);
+  virtual SV *FIRST(ospv_bridge*);
+  virtual SV *NEXT(ospv_bridge*);
   virtual void CLEAR();
   virtual int _count();
 };
